% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util-testing.R
\name{class0}
\alias{class0}
\title{Extract class as a single string.}
\usage{
class0(x)
}
\arguments{
\item{x}{any object.}
}
\description{
Extract class as a single string.
}
