% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage.R
\name{addin_extract_covr}
\alias{addin_extract_covr}
\title{Add-in for Extract & Coverage}
\usage{
addin_extract_covr()
}
\description{
Add-in for Extract & Coverage
}
