% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage.R
\name{addin_covr_file}
\alias{addin_covr_file}
\title{Add-in for \code{covr_file}}
\usage{
addin_covr_file()
}
\description{
This allows for \link{covr_file} to be run from a menu in RStudio.
}
