% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_tests.R
\name{addin_test}
\alias{addin_test}
\alias{addin_test_file}
\title{RStudio add-ins}
\usage{
addin_test()

addin_test_file()
}
\description{
RStudio add-ins
}
